/* PSPP - computes sample statistics.
   Copyright (C) 1997 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   The author may be contacted at <pfaffben@pilot.msu.edu> on the
   Internet, or as Ben Pfaff, 12167 Airport Rd, DeWitt MI 48820, USA
   through more mundane means. */

#if !format_h
#define format_h 1

/* Display format types. */
/* See the definitions of these functions and variables when modifying
   this list:
   misc.c:formats[]
   misc.c:convert_fmt_ItoO()
   sfm-read.c:parse_format_spec()
   sfm-write.c:write_format_spec()
   data-in.c:parse_string_as_format()
   data-out.c:convert_format_to_string(). */
enum
  {
    /* Numeric and string formats. */
    FMT_F,
    FMT_N,
    FMT_E,
    FMT_COMMA,
    FMT_DOT,
    FMT_DOLLAR,
    FMT_PCT,
    FMT_Z,
    FMT_A,
    FMT_AHEX,
    FMT_IB,
    FMT_P,
    FMT_PIB,
    FMT_PIBHEX,
    FMT_PK,
    FMT_RB,
    FMT_RBHEX,

    /* Custom currency. */
    FMT_CCA,
    FMT_CCB,
    FMT_CCC,
    FMT_CCD,
    FMT_CCE,

    /* Date/time formats. */
    FMT_DATE,
    FMT_EDATE,
    FMT_SDATE,
    FMT_ADATE,
    FMT_JDATE,
    FMT_QYR,
    FMT_MOYR,
    FMT_WKYR,
    FMT_DATETIME,
    FMT_TIME,
    FMT_DTIME,
    FMT_WKDAY,
    FMT_MONTH,
    
    /* These aren't real formats.  They're used by DATA LIST. */
    FMT_T,
    FMT_X,
    FMT_DESCEND,
    FMT_NEWREC,
    
    FMT_NUMBER_OF_FORMATS,
  };

/* Describes one of the display formats above. */
typedef struct
  {
    char name[9];		/* `DATETIME' is the longest. */
    int n_args;			/* 1=width; 2=width.decimals. */
    int Imin_w, Imax_w;		/* Limits on input width. */
    int Omin_w, Omax_w;		/* Limits on output width. */
    int cat;			/* Category. */
    int output;			/* Output format. */
  }
fmt_desc;

/* Display format categories. */
enum
  {
    FCAT_BLANKS_SYSMIS = 001,	/* 1=All-whitespace means SYSMIS. */
    FCAT_EVEN_WIDTH = 002,	/* 1=Width must be even. */
    FCAT_STRING = 004,		/* 1=String input/output format. */
    FCAT_SHIFT_DECIMAL = 010,	/* 1=Automatically shift decimal point
				   on output--used for fixed-point
				   formats. */
    FCAT_OUTPUT_ONLY = 020,	/* 1=This is not an input format. */
  };

/* Display format. */
typedef struct fmt_spec
  {
    int type;			/* One of the above constants. */
    int w;			/* Width. */
    int d;			/* Number of implied decimal places. */
  }
fmt_spec;

/* Descriptions of all the display formats above. */
extern fmt_desc formats[];

/* Set this flag to cause errors to return SYSMIS but not report an
   error to the user. */
extern int fmt_parse_ignore_error;

union value;

int parse_format_specifier (fmt_spec *input, int allow_xt);
int parse_format_specifier_name (const char **cp, int allow_xt);
int check_input_specifier (const fmt_spec *spec);
int check_output_specifier (const fmt_spec *spec);
int check_string_specifier (const fmt_spec *spec, int min_len);
void convert_fmt_ItoO (const fmt_spec *input, fmt_spec *output);
int parse_string_as_format (const char *s, int len, const fmt_spec *fp,
			    int fc, union value *v);
int convert_format_to_string (char *s, const fmt_spec *fp,
			      const union value *v);
char *fmt_to_string (const fmt_spec *);
void num_to_string (double v, char *s, int w, int d);

#endif /* !format_h */
